/*
 * DrawPanel3D.java
 *
 * Created on 28. September 2007, 15:23
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

import Engine.*;
import Engine.Primitives.*;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.JPanel;

/**
 *
 * @author Sunshine
 */
public class DrawPanel3D extends JPanel {
    
    int height;
    int width;
    SkyscraperApplet applet;
    
    Engine engine;
    Quad ground;
    
    // the colored planes showing us from which side we look at the scenary
    Quad frontside, leftside, rightside, backside; 

    SkyScraper[][] skyscrapers;         // our array of our skyscrapers
    int n;          // width and height of grid
    
    float rotAngleY;
    float rotAngleX;
    
    /** Creates a new instance of DrawPanel3D */
    public DrawPanel3D(int width, int height, SkyscraperApplet appl) 
    {
        this.setBackground(Color.WHITE);
        this.setLayout(null);
        this.setBounds(0,0,width, height);
        this.width = width;
        this.height = height;
        this.applet = appl;
        
        this.setBorder(new javax.swing.border.TitledBorder(""));
        this.setVisible(true);
        
        this.setFocusable(true);
        this.requestFocus();
        
        InitEngine();
        
        // add key handling
        this.addKeyListener(new KeyAdapter() {
            public void keyPressed(KeyEvent e) 
            {
                if (e.getKeyCode() == KeyEvent.VK_A)    // rotate world left
                {
                    rotAngleY += 0.04;
                    ground.rotY(0.04);
                    frontside.rotY(0.04);
                    leftside.rotY(0.04);
                    rightside.rotY(0.04);
                    backside.rotY(0.04);
                    for (int x = 0; x < n; x++)
                        for (int y = 0; y < n; y++)
                            skyscrapers[x][y].rotY(0.04);
                }
                else if (e.getKeyCode() == KeyEvent.VK_S) // rotate world right
                {
                    rotAngleY -= 0.04;
                    ground.rotY(-0.04);
                    frontside.rotY(-0.04);
                    leftside.rotY(-0.04);
                    rightside.rotY(-0.04);
                    backside.rotY(-0.04);
                    for (int x = 0; x < n; x++)
                        for (int y = 0; y < n; y++)
                            skyscrapers[x][y].rotY(-0.04);
                }
                else if (e.getKeyCode() == KeyEvent.VK_UP)  // move camera up
                    engine.cameraPos.y += 5;                
                else if (e.getKeyCode() == KeyEvent.VK_DOWN)  // move camera down
                    engine.cameraPos.y -= 5;  
                else if (e.getKeyCode() == KeyEvent.VK_LEFT)  // move camera down
                    engine.cameraPos.x -= 5;
                else if (e.getKeyCode() == KeyEvent.VK_RIGHT)  // move camera down
                    engine.cameraPos.x += 5;
                else if (e.getKeyCode() == KeyEvent.VK_M)  // move camera up
                    engine.cameraPos.z += 5;
                else if (e.getKeyCode() == KeyEvent.VK_N)  // move camera down
                    engine.cameraPos.z -= 5;
                else if (e.getKeyCode() == KeyEvent.VK_G)  // rot camera about its x-axis
                {    if (engine.camAngleX < 0.5) engine.camAngleX += 0.05; }
                else if (e.getKeyCode() == KeyEvent.VK_B)  // rot camera about its x-axis
                {    if (engine.camAngleX > -0.3) engine.camAngleX -= 0.05; }
                
                repaint();
                setFocusable(true);
            }
            
        });
        
        // handle mouse click to be sure to gain focus
        this.addMouseListener(new MouseAdapter() 
        {
            public void mouseClicked(MouseEvent evt) 
            { 
                setFocusable(true);
                requestFocusInWindow();
            }
        });
    }
    
    /*
     * Init the 3d view and the whole world.
     */
    private void InitEngine()
    {
        rotAngleY = 0;
        rotAngleX = 0;
        
        engine = new Engine(width, height);
        // move camera back
        engine.cameraPos.z = -230;
        
        // init the ground
        ground = new Quad(      new Vector3D(-100, 0, -100),
                                new Vector3D(100, 0, -100),
                                new Vector3D(100, 0, 100),
                                new Vector3D(-100, 0, 100));
        // set the ground in the middle of the world
        ground.worldPos = new Vector3D(0, -100, 0);
        engine.add(ground);
        
        // add frontside help-plane
        frontside = new Quad(   new Vector3D(-100, 0, -110),
                                new Vector3D(100, 0, -110),
                                new Vector3D(100, 0, -115),  
                                new Vector3D(-100, 0, -115),
                                GameField.frontColor);
        // add leftside help-plane
        leftside = new Quad(   new Vector3D(-110, 0, -100),
                                new Vector3D(-110, 0, 100),
                                new Vector3D(-115, 0, 100),  
                                new Vector3D(-115, 0, -100),
                                GameField.leftColor);
        // add rightside help-plane
        rightside = new Quad(   new Vector3D(110, 0, -100),
                                new Vector3D(110, 0, 100),
                                new Vector3D(115, 0, 100),  
                                new Vector3D(115, 0, -100),
                                GameField.rightColor);
        // add rightside help-plane
        backside = new Quad(   new Vector3D(-100, 0, 110),
                                new Vector3D(-100, 0, 115),
                                new Vector3D(100, 0, 115),  
                                new Vector3D(100, 0, 110),
                                GameField.backColor);
        
        frontside.worldPos = ground.worldPos;
        leftside.worldPos = ground.worldPos;
        rightside.worldPos = ground.worldPos;
        backside.worldPos = ground.worldPos;
        engine.add(frontside);
        engine.add(leftside);
        engine.add(rightside);
        engine.add(backside);
        // to do: better camera adjusting!
        engine.cameraPos = new Vector3D(0,30,-230);
        engine.camAngleX += 0.2;

    }
    
    
     /*
     * Here goes all the painting
     */
    public void paintComponent(Graphics g)
    {
        super.paintComponent(g);
        if (!applet.bFileLoaded) return;        // no quest loaded so nothing to draw
        engine.render(g);
    }
    
    /*
     * On loading new quest, call this to init engine with the new quest.
     */
    public void setNewQuest(byte[][] field)
    {
        // delete renderlist
        engine.clearAll();
        // build new skyscraper array
        n = applet.quest.size;
        initSkyScrapers(field, n);
        // add all objects again to renderlist
        engine.add(ground);
        engine.add(frontside);
        engine.add(leftside);
        engine.add(rightside);
        engine.add(backside);
        for (int x = 0; x < n; x++)
            for (int y = 0; y < n; y++)
                engine.add(skyscrapers[x][y]);

        for (int x = 0; x < n; x++)
            for (int y = 0; y < n; y++)
            {
                skyscrapers[x][y].rotY(rotAngleY);
                skyscrapers[x][y].rotX(rotAngleX);
            }
        repaint();
    }
    
    /*
     * Init our array with skyscrapers
     */
    public void initSkyScrapers(byte[][] field, int size)
    {
        // init array
        this.skyscrapers = new SkyScraper[size][size];
        // calculate width/length of one skyscraper; ground size is fixed at 200
        int off = 200 / size;
        
        // define 150 as biggest height of a skyscraper, 
        int stepheight = 150 / size;
        
        // temporary skyscs
        Quad left, front, right, back;
        
        int h;
        for (int x = 0; x < size; x++)
        {
            for (int y = 0; y < size; y++)
            {
                //if (field[x][y] == 0) continue;
                h = stepheight * field[x][y];
                front = new Quad(   new Vector3D(-100 + y*off,          0, 100 -off - x*off),
                                    new Vector3D(-100 + off + y*off,    0, 100 -off - x*off),
                                    new Vector3D(-100 + off + y*off,    h, 100 -off - x*off),
                                    new Vector3D(-100 + y*off,          h, 100 -off - x*off) ); 
                
                left = new Quad(    new Vector3D(-100 + y*off, 0,       100 - off - x*off),
                                    new Vector3D(-100 + y*off, 0,       100 - x*off),
                                    new Vector3D(-100 + y*off, h,       100 - x*off),
                                    new Vector3D(-100 + y*off, h,       100 - off - x*off) );

                right = new Quad(   new Vector3D(-100 + off + y*off, 0, 100 - x*off),
                                    new Vector3D(-100 + off + y*off, 0, 100 - off - x*off),
                                    new Vector3D(-100 + off + y*off, h, 100 - off - x*off),
                                    new Vector3D(-100 + off + y*off, h, 100 - x*off) );

                back = new Quad(   new Vector3D(-100 + y*off,        0, 100 - x*off),
                                    new Vector3D(-100 + off + y*off, 0, 100 - x*off),
                                    new Vector3D(-100 + off + y*off, h, 100 - x*off),
                                    new Vector3D(-100 + y*off,       h, 100 - x*off) );
                
                skyscrapers[x][y] = new SkyScraper(front, left, right, back, field[x][y], size);
            }
        }
        
        /* this is important - to rotating the ground and all skyscrapers together,
         * we set the worldpos the same - so the worldpos of ALL skyscrapers is the middlepoint
         * of the groundquad, so the skysc were defined depending on this point
         */
        SkyScraper.worldPos = ground.worldPos;
    }
    
    /*
     * update a skyscraper if user changed the height with his input.  DOES NOT WORK!
     * IDEA: give each skysc an ID and search for & replace it explizit - TO DO!
     */
    /*
    public void updateCell(int x, int y, int value)
    {
        Quad left, front, right, back;
        
        int off = 200 / n;
        int stepheight = 150 / n;
        
        int h = stepheight * applet.field.game[x][y];
        front = new Quad(   new Vector3D(-100 + x*off,          0,        -100 + y*off),
                            new Vector3D(-100 + off + x*off,    0,        -100 + y*off),
                            new Vector3D(-100 + off + x*off,    h, -100 + y*off),
                            new Vector3D(-100 + x*off,          h, -100 + y*off) ); 

        left = new Quad(    new Vector3D(-100 + x*off, 0,       -100 + off + y*off),
                            new Vector3D(-100 + x*off, 0,       -100 + y*off),
                            new Vector3D(-100 + x*off, h, -100 + y*off),
                            new Vector3D(-100 + x*off, h, -100 + off + y*off) );

        right = new Quad(   new Vector3D(-100 + off + x*off, 0,         -100 + y*off),
                            new Vector3D(-100 + off + x*off, 0,         -100 + off + y*off),
                            new Vector3D(-100 + off + x*off, h, -100 + off + y*off),
                            new Vector3D(-100 + off + x*off, h, -100 + y*off) );

        back = new Quad(   new Vector3D(-100 + x*off,        0,        -100 + off + y*off),
                            new Vector3D(-100 + off + x*off, 0, -100 + off + y*off),
                            new Vector3D(-100 + off + x*off, h, -100 + off + y*off),
                            new Vector3D(-100 + x*off,       h, -100 + off + y*off) );
                            
        SkyScraper skytmp = new SkyScraper(front, left, right, back, applet.field.game[x][y], n);
        // del
        engine.replace(skyscrapers[x][y], skytmp);
        skyscrapers[x][y] = skytmp;
        repaint();
    }
    */
    
}
