/*
 * Engine.java
 *
 * Created on 2. Mrz 2007, 12:58
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package Engine;

import Engine.Primitives.*;
import java.awt.Graphics;
import java.util.*;
import java.awt.Point;

/**
 *
 * @author Sunshine
 *
 * Till now, we assume camera is at (0,0,0) at we look down the positive z-axis
 *
 */
public class Engine {

    public static final byte MODE_PERSPECTIVE = 0;
    public static final byte MODE_ORTHO = 1;

    public byte mode;              // projection mode - persp. or ortho

    int width, height, width2, height2;
    int SCALE = 180;

    // camera stuff
    public Vector3D cameraPos;
    public double camAngleX, camAngleY, camAngleZ;

    ArrayList<PrimitiveInterface> list = new ArrayList();

    /** Creates a new instance of Engine */
    public Engine(int width, int height) {
        this.width = width;
        this.height = height;
        this.width2 = width / 2;
        this.height2 = height / 2;
        cameraPos = new Vector3D(0,0,0);
        camAngleX = camAngleY = camAngleZ = 0;
        mode = MODE_PERSPECTIVE;
    }

    /*
     * Add a triangle to render object list
     */
    public void addTriangle(Vector3D triangle[])
    {
        list.add(new Triangle(triangle));
    }

    public void add(PrimitiveInterface obj)
    {
        list.add(obj);
    }

    /*
     * Add a quad to render object list
     */
    public void addQuad(Quad quad)
    {
        list.add(quad);
    }

    /*
     * Delete all objects from render list
     */
    public void clearAll() {
        list.clear();
        int ind = list.size();
    }

    public void replace(PrimitiveInterface oldObj, PrimitiveInterface newObj)
    {
        int index = list.indexOf(oldObj);
        list.set(index, newObj);
    }


    /*
     * Render all objects
     */
    public void render(Graphics g)
    {
        g.clearRect(0,0,width,height);
        PrimitiveInterface currentObject;
        Vector3D vertices[];
        Vector3D worldPos;
        int numVertices;

        // temp variables
        Vector3D worldCoords[];// = new Vector3D[4];
        Vector3D cameraCoords[];
        Vector3D perCoords[];// = new Vector3D[4];
        Point screenCoords[];// = new Point[4];

        for (int j = 0; j < list.size(); j++)
        {
            // get current object
            currentObject = list.get(j);
            // get vertices
            vertices = currentObject.getVertices();
            // get position in world
            worldPos = currentObject.getWorldPos();
            // get number of vertices
            numVertices = currentObject.getNumOfVertices();

            worldCoords = new Vector3D[numVertices];
            cameraCoords = new Vector3D[numVertices];
            perCoords = new Vector3D[numVertices];
            screenCoords = new Point[numVertices];
            boolean bBehindCam = false;
            for (int i = 0; i < numVertices; i++)
            {
                worldCoords[i] = new Vector3D();
                cameraCoords[i] = new Vector3D();
                perCoords[i] = new Vector3D();
                screenCoords[i] = new Point();

                // transform model to world coordinates
                worldCoords[i].x = vertices[i].x + worldPos.x;
                worldCoords[i].y = vertices[i].y + worldPos.y;
                worldCoords[i].z = vertices[i].z + worldPos.z;

                // transform world to camera coordinates
                cameraCoords[i].x = worldCoords[i].x - cameraPos.x;
                cameraCoords[i].y = worldCoords[i].y - cameraPos.y;
                cameraCoords[i].z = worldCoords[i].z - cameraPos.z;
                cameraCoords[i].rotY(-camAngleY);
                cameraCoords[i].rotX(-camAngleX);
                cameraCoords[i].rotZ(-camAngleZ);

                if (cameraCoords[i].z < 0) bBehindCam = true;
                if (cameraCoords[i].z > 900) bBehindCam = true;
                //if ((cameraCoords[i].x) < cameraCoords[i].z) bBehindCam = true;

                // transform world to perspective models
                /*perCoords[i].x = cameraCoords[i].x / cameraCoords[i].z;
                perCoords[i].y = cameraCoords[i].y / cameraCoords[i].z;
                perCoords[i].z = cameraCoords[i].z;

                // from perspective to screen
                screenCoords[i].x = (int)((perCoords[i].x+1)*(0.5 * width - 0.5));
                screenCoords[i].y = (int)((height-1) - (perCoords[i].y+1)*(0.5 * height - 0.5));
                */

                if (mode == MODE_PERSPECTIVE)
                {
                    screenCoords[i].x = (int)(cameraCoords[i].x *SCALE / cameraCoords[i].z + width2);
                    screenCoords[i].y = (int)(-cameraCoords[i].y *SCALE / cameraCoords[i].z + height2);
                }
                else        // MODE_ORTHOGONAL
                {
                    screenCoords[i].x = (int)(cameraCoords[i].x  + width2);
                    screenCoords[i].y = (int)(-cameraCoords[i].y + height2);
                }

            }

            // now render
            if (!bBehindCam)
                currentObject.render(g, screenCoords);
        }
    }

}
