/*
 * Quad.java
 *
 * Created on 2. Mrz 2007, 14:18
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package Engine.Primitives;

import Engine.*;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;

/**
 *
 * @author Sunshine
 */
public class Quad implements PrimitiveInterface {
    
    public Vector3D vertices[];
    public Vector3D worldPos;
    
    Color color = null;
    
    /** Creates a new instance of Quad */
    public Quad(Vector3D quad[]) {
        vertices = new Vector3D[4];
        if (quad.length != 3) return;
        vertices[0] = quad[0];
        vertices[1] = quad[1];
        vertices[2] = quad[2];
        vertices[3] = quad[3];
        worldPos = new Vector3D(0,0,100);
    }
    
    public Quad(Vector3D v0, Vector3D v1, Vector3D v2, Vector3D v3)
    {
        vertices = new Vector3D[4];
        vertices[0] = v0;
        vertices[1] = v1;
        vertices[2] = v2;
        vertices[3] = v3;
        worldPos = new Vector3D(0,0,100);
    }
    
    public Quad(Vector3D v0, Vector3D v1, Vector3D v2, Vector3D v3, Color c)
    {
        vertices = new Vector3D[4];
        vertices[0] = v0;
        vertices[1] = v1;
        vertices[2] = v2;
        vertices[3] = v3;
        worldPos = new Vector3D(0,0,100);
        color = c;
    }
    
    
    // assume camera is at (0,0,0)
    // pos of quad in world is (0, 0, 100)
    
     public void render(Graphics g, Point[] screenCoords) 
     {
         Color oldColor = g.getColor(); 
         if (color != null) g.setColor(color);
         g.drawLine(screenCoords[0].x, screenCoords[0].y,
                    screenCoords[1].x, screenCoords[1].y);
         g.drawLine(screenCoords[1].x, screenCoords[1].y,
                    screenCoords[2].x, screenCoords[2].y);
         g.drawLine(screenCoords[2].x, screenCoords[2].y,
                    screenCoords[3].x, screenCoords[3].y);
         g.drawLine(screenCoords[3].x, screenCoords[3].y,
                    screenCoords[0].x, screenCoords[0].y);
         g.setColor(oldColor);
     }
     
     /*
      * Render the quad with 4 vertices starting from startIndex from given screenCoords
      */
 public void render(Graphics g, Point[] screenCoords, int startIndex, Color color) 
     {
         g.setColor(color);
         g.drawLine(screenCoords[startIndex].x, screenCoords[startIndex].y,
                    screenCoords[startIndex+1].x, screenCoords[startIndex+1].y);
         g.drawLine(screenCoords[startIndex+1].x, screenCoords[startIndex+1].y,
                    screenCoords[startIndex+2].x, screenCoords[startIndex+2].y);
         g.drawLine(screenCoords[startIndex+2].x, screenCoords[startIndex+2].y,
                    screenCoords[startIndex+3].x, screenCoords[startIndex+3].y);
         g.drawLine(screenCoords[startIndex+3].x, screenCoords[startIndex+3].y,
                    screenCoords[startIndex+0].x, screenCoords[startIndex+0].y);
         g.setColor(Color.BLACK);
     }
     
     public void renderSolid(Graphics g, Point[] screenCoords, int startIndex, Color color) 
     {
         // first convert given coordinates in format required by fillpolygon function
         // senseless overhead :-(
         int[] xpoints = new int[4];
         int[] ypoints = new int[4];
         for (int i = 0; i < 4; i++)
         {
             xpoints[i] = screenCoords[startIndex+i].x;
             ypoints[i] = screenCoords[startIndex+i].y;
         }
         Color oldColor = g.getColor();
         g.setColor(color);
         g.fillPolygon(xpoints, ypoints, 4);
        
         g.setColor(oldColor);
     }
     
     public int getNumOfVertices() { return 4; }
     public Vector3D[] getVertices() { return vertices; }
     public Vector3D getWorldPos() { return worldPos; }
     
     
     
     
     
     public void rotY(double angle) {
         for (int i = 0; i < 4; i++)
             vertices[i].rotY(angle);
     }
     public void rotX(double angle) {
         for (int i = 0; i < 4; i++)
             vertices[i].rotX(angle);
     }
     public void rotZ(double angle) {
         for (int i = 0; i < 4; i++)
             vertices[i].rotZ(angle);
     }
    
}
