import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
/*
 * GameField.java
 *
 * Created on 27. September 2007, 19:41
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

/**
 *
 * @author Sunshine
 */
public class GameField extends javax.swing.JPanel {
    
    public static final Color frontColor = Color.BLUE;
    public static final Color leftColor = new Color(100, 200, 255);//Color.CYAN;
    public static final Color rightColor = Color.GREEN;
    public static final Color backColor = Color.GRAY;
    
    SkyscraperApplet applet;
    
    int numOfCells;         // num of cells
    int gridsize;           // width/height of one cell in pixel
    byte[][] game = null;   // our gamefield with user inputs
    
    int currentCellX = -1, currentCellY = -1;  
    
    /* for showing errors */
    boolean[][] wrongCells = null;
    boolean[][] wrongVisSkyscs = null;
    boolean bShowErrors = false;
            
    /** Creates a new instance of GameField */
    public GameField(int width, int height, SkyscraperApplet applet) 
    {
        this.applet = applet;
        this.setLayout(null);
        this.setBounds(0, 0, width, height);
        this.setBorder(new javax.swing.border.TitledBorder(""));     
        
        this.addKeyListener(new KeyAdapter() 
        {
            public void keyPressed(KeyEvent evt)
            {
                HandleKeys(evt);
            }
        });
        
        this.addMouseListener(new MouseAdapter() 
        {
            public void mouseClicked(MouseEvent evt) { HandleMouse(evt); }
        });
        this.addMouseMotionListener(new MouseAdapter() 
        {
            public void mouseMoved(MouseEvent evt) { HandleMouse(evt); }
        });
        
        this.setFocusable(true);    // we need the focus the get user input!!!
        this.requestFocus();
    }
    
  
    /*
     * Set new quest. The given visible skyscrapers don't change so we make no copy
     * of them but just read them from the quest class.
     */
    public void setField(byte[][] field)
    {
        numOfCells = applet.quest.size;
        game = new byte[numOfCells][numOfCells];
        for (int i = 0; i < numOfCells; i++)
            for (int j = 0; j < numOfCells; j++)
                game[i][j] = field[i][j];
    }
    
    /*
     * Handle pressed keys
     */
    public void HandleKeys(KeyEvent evt)
    {
        if (!applet.bFileLoaded) return; 
        if (currentCellX == -1 || currentCellY == -1) return;
        
        char key = evt.getKeyChar();
        // process normal number keys
       	if ( (key >= '0') && (key <= '1' + numOfCells-1) )
        {
            game[currentCellX-1][currentCellY-1] = (byte)(key - '1' + 1);
            repaint();
            isQuestSolved();        // check if user solved quest
            // update 3D view
            applet.update3DView(currentCellX-1, currentCellX-1, game[currentCellX-1][currentCellY-1]);
        }
        this.setFocusable(true);
    }
    
    /*
     * Handle when user clicked mouse button
     */
    public void HandleMouse(MouseEvent evt)
    {
        if (!applet.bFileLoaded || gridsize == 0) return; 
        // get cell, note that here I had a major bug!
        // it was mirrored -> error; fixed by mixing next two lines so the x and y are right!!!
        int cellx = evt.getY() / gridsize;
        int celly = evt.getX() / gridsize;
        if (cellx >= 1 && cellx <= numOfCells && celly >= 1 && celly <= numOfCells)
        {
            currentCellX = cellx;
            currentCellY = celly;
            // first check if value is a given = fixed one; if yes, do nothing
            if (applet.quest.field[currentCellX-1][currentCellY-1] != 0) return;
            // by rightclick delete current value
            if (evt.getButton() == MouseEvent.BUTTON3) 
            {
                game[currentCellX-1][currentCellY-1] = 0;
                // update 3D view
                applet.update3DView(currentCellX-1, currentCellX-1, game[currentCellX-1][currentCellY-1]);
            }
            // on leftclick increase cell value
            else if (evt.getButton() == MouseEvent.BUTTON1)
            {
                game[currentCellX-1][currentCellY-1]++;
                if (game[currentCellX-1][currentCellY-1] > numOfCells)
                    game[currentCellX-1][currentCellY-1] = 0;
                isQuestSolved();
                // update 3D view
                applet.update3DView(currentCellX-1, currentCellX-1, game[currentCellX-1][currentCellY-1]);
            }
            repaint(); 
        }
        this.setFocusable(true);
        this.requestFocusInWindow();
    }
    
    /*
     * Check if user solved the game
     */
    private boolean isQuestSolved()
    {
        if ((applet.quest.checkRowsandColumns_fast(game) &&
            applet.quest.checkVisibility(game)))
        {
            applet.QuestSolved();
            return true;
        }
        return false;
    }
    
    /*
     * Here goes all the painting
     */
    public void paintComponent(Graphics g)
    {
        super.paintComponent(g);
        if (!applet.bFileLoaded) return;        // no quest loaded
        
        // calculate field size
        int n = applet.quest.size;
        gridsize = (this.getSize().height) / (n+2);
        // paint the grid
        for (int i = 0; i <= n; i++)
        {
            // horizontal lines
            g.drawLine(gridsize, gridsize*(i+1), gridsize*(n+1), gridsize*(i+1));
            // vertical lines
            g.drawLine(gridsize + gridsize*i, gridsize, gridsize*(i+1), gridsize*(n+1));
        }
        // make the outer lines thicker and in special color
        g.setColor(GameField.backColor);
        g.fillRect(gridsize, gridsize-3, gridsize*n, 3);
        g.setColor(GameField.leftColor);
        g.fillRect(gridsize-3, gridsize, 3, gridsize*n);
        g.setColor(GameField.rightColor);
        g.fillRect(gridsize*(n+1), gridsize, 3, gridsize*n);
        g.setColor(GameField.frontColor);
        g.fillRect(gridsize, gridsize*(n+1), gridsize*n, 3);
        
        // if user want to see the errors, we have to update our error structure
        if (bShowErrors) this.updateErrors();
        
        g.setColor(Color.BLACK);
        // insert the quest
        if (game == null) return;       // should not happen
        int xPos, yPos;
        Font font = new Font("Arial", Font.PLAIN, gridsize / 2);
        g.setFont(font);
        FontMetrics fm = g.getFontMetrics(font);
        for (int i = 0; i < n; i++)
            for (int j = 0; j < n; j++)
            {
                // don't draw a zero
                if (game[i][j] == 0 ) continue;
                java.awt.geom.Rectangle2D rect = fm.getStringBounds(Integer.toString(game[i][j]), g);
                int textHeight = (int)(rect.getHeight()); 
                int textWidth  = (int)(rect.getWidth());

                // Center text horizontally and vertically
                xPos = gridsize*(j+1) + (gridsize  - textWidth)  / 2;
                yPos = gridsize*(i+1) + (gridsize - textHeight) / 2  + fm.getAscent();
                
                // paint erros red
                if (this.bShowErrors && this.wrongCells[i][j]) g.setColor(Color.RED);
                g.drawString(Integer.toString(game[i][j]), xPos, yPos);
                g.setColor(Color.BLACK);
            }
        
        // insert the visible skyscrapers info
        byte[][] visibleSkyscs = applet.quest.visibleTowers;
        
        for (int i = 0; i < 4; i++)
            for (int j = 0; j < n; j++)
            {
                if (visibleSkyscs[i][j] == 0) continue;
                java.awt.geom.Rectangle2D rect = fm.getStringBounds(Integer.toString(visibleSkyscs[i][j]), g);
                int textHeight = (int)(rect.getHeight()); 
                int textWidth  = (int)(rect.getWidth());

                // Center text horizontally and vertically
                if (i == Quest.TOP)
                {
                    xPos = gridsize*(j+1) + (gridsize  - textWidth)  / 2;
                    yPos = (gridsize - textHeight) / 2  + fm.getAscent();
                }
                else if (i == Quest.BOTTOM)
                {
                    xPos = gridsize*(j+1) + (gridsize  - textWidth)  / 2;
                    yPos = gridsize*(n+1) + (gridsize - textHeight) / 2  + fm.getAscent();
                }
                else if (i == Quest.LEFT)
                {
                    xPos = (gridsize  - textWidth)  / 2;
                    yPos = gridsize*(j+1) + (gridsize - textHeight) / 2  + fm.getAscent();
                }
                else
                {
                    xPos = gridsize*(n+1) + (gridsize  - textWidth)  / 2;
                    yPos = gridsize*(j+1) + (gridsize - textHeight) / 2  + fm.getAscent();
                }
                // paint the value; if error enabled and error found, draw it in red
                if (this.bShowErrors && this.wrongVisSkyscs[i][j]) g.setColor(Color.RED);
                g.drawString(Integer.toString(visibleSkyscs[i][j]), xPos, yPos);
                g.setColor(Color.BLACK);
            }
        
        // draw current cell
        g.setColor(Color.RED);
        g.drawRect(gridsize * currentCellY, gridsize * currentCellX, gridsize, gridsize);
    }
    
    
    
    
    
    
    /********************************************************
     **** GET ALL WRONG VALUES AND SHOW THEM ****************
     *******************************************************/
    
    /* I made it easy and copied just (slightly modified) the functions from Quest class. */
    
    /*
     * Enables/disables if erros should be drawn in red or not. 
     */
    public void toggleErrors()
    {
        if (bShowErrors)
        {
            bShowErrors = false;
            repaint(); 
            return;
        }
        // here enable show errors
        this.bShowErrors = true;
        repaint();
    }
    
    private void updateErrors()
    {
        this.wrongCells = new boolean[numOfCells][numOfCells];
        for (int i = 0; i < numOfCells; i++)
            for (int j = 0; j < numOfCells; j++)
                this.wrongCells[i][j] = false;
        this.wrongVisSkyscs = new boolean[4][numOfCells];
        for (int i = 0; i < 4; i++)
            for (int j = 0; j < numOfCells; j++)
                this.wrongVisSkyscs[i][j] = false;
        
        this.getRowsAndColumnsErrors();
        this.getVisibilityErrors();
    }
             
    
    private void getRowsAndColumnsErrors()
    {
        ArrayList<Byte> list1 = new ArrayList<Byte>();
        ArrayList<Byte> list2 = new ArrayList<Byte>();
        for (int x = 0; x < numOfCells; x++)
        {
            list1.clear();
            list2.clear();
            for (int y = 0; y < numOfCells; y++)
            {
                if (game[x][y] != 0)
                {
                    if (list1.contains(game[x][y]))
                    {
                        for (int i = 0; i < numOfCells; i++)
                            if (game[x][i] == game[x][y])
                                this.wrongCells[x][i] = true; 
                    }  
                    else
                        list1.add(game[x][y]);
                }
                
                if (game[y][x] != 0)
                {
                    if (list2.contains(game[y][x]))
                    {
                        for (int i = 0; i < numOfCells; i++)
                            if (game[i][x] == game[y][x])
                                this.wrongCells[i][x] = true;
                    }    
                    else
                        list2.add(game[y][x]);
                }
            }
        }   
    }
    
    
    private void getVisibilityErrors()
    {
        boolean bErrorFound = false;
        // loop through all rows and columns
        byte[][] visibleTowers = applet.quest.visibleTowers;
        for (int i = Quest.TOP; i <= Quest.BOTTOM; i++)
        {
            for (int j = 0; j < numOfCells; j++)
            {
                // no number given so continue
                if (visibleTowers[i][j] == 0) continue;
                
                // ok so check if we see as many skyscrapers as it should be
                int numOfVisable = 0;
                int heighest = -1;
                if (i == Quest.TOP)           // TOP
                {
                    // we have to go from top to bottom in current column
                    for (int k = 0; k < numOfCells; k++)
                    {
                        // if no skyscraper is on this field, go on with next field on column
                        if (game[k][j] == 0) continue;
                        if (game[k][j] > heighest)
                        {
                            numOfVisable++;
                            heighest = game[k][j];
                        }
                    }
                    // now check!
                    if (visibleTowers[i][j] != numOfVisable)  wrongVisSkyscs[i][j] = true;
                }
                
                else if (i == Quest.LEFT)     // LEFT
                {
                    // we have to go from left to right in current r
                    for (int k = 0; k < numOfCells; k++)
                    {
                        // if no skyscraper is on this field, go on with next field on column
                        if (game[j][k] == 0) continue;
                        if (game[j][k] > heighest)
                        {
                            numOfVisable++;
                            heighest = game[j][k];
                        }
                    }
                    // now check!
                    if (visibleTowers[i][j] != numOfVisable) wrongVisSkyscs[i][j] = true;
                }
                
                else if (i == Quest.RIGHT)        // RIGHT
                {
                    // we have to go from right to left in current r
                    for (int k = numOfCells-1; k >= 0; k--)
                    {
                        // if no skyscraper is on this field, go on with next field on column
                        if (game[j][k] == 0) continue;
                        if (game[j][k] > heighest)
                        {
                            numOfVisable++;
                            heighest = game[j][k];
                        }
                    }
                    // now check!
                    if (visibleTowers[i][j] != numOfVisable) wrongVisSkyscs[i][j] = true;
                }
                
                if (i == Quest.BOTTOM)           // BOTTOM
                {
                    // we have to go from top to bottom in current column
                    for (int k = numOfCells-1; k >= 0; k--)
                    {
                        // if no skyscraper is on this field, go on with next field on column
                        if (game[k][j] == 0) continue;
                        if (game[k][j] > heighest)
                        {
                            numOfVisable++;
                            heighest = game[k][j];
                        }
                    }
                    
                    // now check!
                    if (visibleTowers[i][j] != numOfVisable) wrongVisSkyscs[i][j] = true;
                }
                
            }
        }
        return;
    }
}
