import java.io.BufferedReader;
import java.io.InputStreamReader;
/*
 * Quest.java
 *
 * Created on 27. September 2007, 18:35
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

/**
 *
 * @author Sunshine
 */

/*
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
*/

public class Quest {
    
    public static final byte TOP = 0;
    public static final byte LEFT = 1;
    public static final byte RIGHT = 2;
    public static final byte BOTTOM = 3;
    
    byte[][]        field;
    byte[][]        visibleTowers;
    int             size;               // width and height of gamefield
    int             size2;              // size^2 = number of cells on gamefield
    
    
    /** Creates a new instance of Quest */
    public Quest() {
    }

    /*
     * Load quest from given file
     */
    public boolean LoadQuestFromFile(String filename)
    {
        BufferedReader in = null;
        try 
        {
            in = new BufferedReader(new InputStreamReader(
                        getClass().getResourceAsStream(filename)));
            String line = null;
            // read first line
            line = in.readLine();
            // from that we know the size of our quest, so we can init our arrays
            size = line.length() - 2;
            size2 = (int)Math.pow(size,2);
            field = new byte[size][size];
            visibleTowers = new byte[4][size];
            // from the first line we know the towers which are visible from the top row
            for (int i = 1; i < line.length() -1; i++)
                visibleTowers[TOP][i-1] = (byte)line.charAt(i);
            
            // now we read in the gamefield; the first and last number in each line is the given number of visible
            // skyscrapers from left/right in the current row
            for (int i = 0; i < size; i++)
            {
                line = in.readLine();
                byte[] b = line.getBytes();
                visibleTowers[LEFT][i] = b[0];
                for (int j = 1; j < line.length() - 1; j++)
                    field[i][j-1] = b[j];
                visibleTowers[RIGHT][i] = b[line.length() - 1];
            }
            
            // at last the last line where we get the visible towers from the bottom row; ignore first and last character
            line = in.readLine();
            for (int i = 1; i < line.length() -1; i++)
                visibleTowers[BOTTOM][i-1] = (byte)line.charAt(i);
            
        } 
        catch (Exception ex)
        {
            System.out.println("Could not load quest -> " + filename);
            return false;
        }
       
        finally 
        { 
            if (in != null) 
                try { 
                    in.close();
                } catch (Exception ex) {
                    ex.printStackTrace();
                } 
        }
        
        // finally convert ascii byte values to numbers
        for (int i = 0; i < size; i++) 
        {
            visibleTowers[TOP][i] -= 48;
            visibleTowers[LEFT][i] -= 48;
            visibleTowers[RIGHT][i] -= 48;
            visibleTowers[BOTTOM][i] -= 48;
            for (int j = 0; j < size; j++) 
            {
                field[i][j] -= 48;
            }
        }
        
        return true;
    }
    
    
    /*
     * Check if in each row and in each column every height exists only once.
     * The field has to be fully filled (without zeros!)
     * If yes it returns true; if an rule is violated it returns false.
     */
    public boolean checkRowsandColumns_fast(byte[][] givenCells)
    {
        int sum1, sum2;
        int correctsum = ((size * (size+1)) / 2);
        for (int x = 0; x < size; x++)
        {
            sum1 = 0;
            sum2 = 0;
            for (int y = 0; y < size; y++)
            {
                sum1 += givenCells[x][y];        // sum in row x
                sum2 += givenCells[y][x];        // sum in column x
            }
            if (sum1 != correctsum || sum2 != correctsum) 
                return false;
        }
        return true;
    }
    
    /*
     * Check here if the given numbers of visible skyscrapers in each row and column are fulfilled.
     * Returns true if current quest does not violate this rule; otherwise false;
     */
    public boolean checkVisibility(byte[][] givenCells)
    {
        boolean bErrorFound = false;
        // loop through all rows and columns
        for (int i = TOP; i <= BOTTOM; i++)
        {
            for (int j = 0; j < size; j++)
            {
                // no number given so continue
                if (this.visibleTowers[i][j] == 0) continue;
                
                // ok so check if we see as many skyscrapers as it should be
                int numOfVisable = 0;
                int heighest = -1;
                if (i == TOP)           // TOP
                {
                    // we have to go from top to bottom in current column
                    for (int k = 0; k < size; k++)
                    {
                        // if no skyscraper is on this field, go on with next field on column
                        if (givenCells[k][j] == 0) continue;
                        if (givenCells[k][j] > heighest)
                        {
                            numOfVisable++;
                            heighest = givenCells[k][j];
                        }
                    }
                    // now check!
                    if (visibleTowers[i][j] != numOfVisable)  bErrorFound = true;
                }
                
                else if (i == LEFT)     // LEFT
                {
                    // we have to go from left to right in current r
                    for (int k = 0; k < size; k++)
                    {
                        // if no skyscraper is on this field, go on with next field on column
                        if (givenCells[j][k] == 0) continue;
                        if (givenCells[j][k] > heighest)
                        {
                            numOfVisable++;
                            heighest = givenCells[j][k];
                        }
                    }
                    // now check!
                    if (visibleTowers[i][j] != numOfVisable) bErrorFound = true;
                }
                
                else if (i == RIGHT)        // RIGHT
                {
                    // we have to go from right to left in current r
                    for (int k = size-1; k >= 0; k--)
                    {
                        // if no skyscraper is on this field, go on with next field on column
                        if (givenCells[j][k] == 0) continue;
                        if (givenCells[j][k] > heighest)
                        {
                            numOfVisable++;
                            heighest = givenCells[j][k];
                        }
                    }
                    // now check!
                    if (visibleTowers[i][j] != numOfVisable) bErrorFound = true;
                }
                
                if (i == BOTTOM)           // BOTTOM
                {
                    // we have to go from top to bottom in current column
                    for (int k = size-1; k >= 0; k--)
                    {
                        // if no skyscraper is on this field, go on with next field on column
                        if (givenCells[k][j] == 0) continue;
                        if (givenCells[k][j] > heighest)
                        {
                            numOfVisable++;
                            heighest = givenCells[k][j];
                        }
                    }
                    
                    // now check!
                    if (visibleTowers[i][j] != numOfVisable) bErrorFound = true;
                }
                
            }
        }
        return (!bErrorFound);
    }
    
    
    // DEBUG
    /*
     * Just print given gamefield out to console.
     */
    public void PrintField()
    {
        // first line
        System.out.print("X");
        for (int i = 0; i < size; i++)
            System.out.print((char)(visibleTowers[TOP][i] + 48));
        System.out.print("X");
        System.out.println("");
        
        // now the gamefield
        for (int i = 0; i < size; i++)
        {
            System.out.print((char)(visibleTowers[LEFT][i] + 48));
            for (int j = 0; j < size; j++)
                System.out.print((char)(field[i][j] + 48));
            System.out.print((char)(visibleTowers[RIGHT][i] + 48));
            System.out.println("");
        }
        
        // last line
        System.out.print("X");
        for (int i = 0; i < size; i++)
            System.out.print((char)(visibleTowers[BOTTOM][i] + 48));
        System.out.print("X");
        System.out.println("");
    }
    
}
