/*
 * SkyscraperApplet.java
 * The Skyscraper game as an applet.
 * Created on 27. September 2007, 18:25
 * Author:  Sunshine
 * website: www.sunshine2k.de
 */

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.Arrays;
import javax.swing.JOptionPane;

public class SkyscraperApplet extends javax.swing.JApplet {
    
    LoadQuestFrame questwin;            // frame for loading quests
    Quest quest;                        // the currently loaded quest
    GameField field;                    // the 2D field which gets user input
    DrawPanel3D panel3D;                // 3D wireframe view of quest
    boolean bFileLoaded = false;        // true if a quest is loaded
    
    int currentQuest;                   // loaded quest (number of quest file, e.g. 3 means "3.txt"
            
    /** Initializes the applet SkyscraperApplet */
    public void init() {
        try {
            java.awt.EventQueue.invokeAndWait(new Runnable() {
                public void run() {
                    initComponents();
                }
            });
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        
        this.setSize(650, 430);
        
        quest = new Quest();
        
        questwin = new LoadQuestFrame(this);
        questwin.setVisible(false);
        
        field = new GameField(300, 300, this);
        this.add(field);
        field.setLocation(10, 40);
        
        panel3D = new DrawPanel3D(300, 300, this);
        panel3D.setLocation(330, 40);
        this.add(panel3D);

        // debug
         LoadQuest(1);
         
        // react on LoadQuest Button 
        this.LoadQuestButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) 
            {
                questwin.setVisible(true);
            }
            
        });
        
        // react on restart button click
        this.RestartButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) 
            {
                int res = JOptionPane.showConfirmDialog(null, 
                        "Do want to restart the level? \n All your inputs will be lost.", 
                        "Restart current quest?", JOptionPane.YES_NO_OPTION);
                if (res == JOptionPane.YES_OPTION) 
                {
                    // restart level
                    bFileLoaded = true;
                    field.setField(quest.field);
                    field.repaint();
                    SolvedLabel.setText("");
                    panel3D.setNewQuest(quest.field);
                    bFileLoaded = true;
                }
            }
        });
        
        // react on next quest button
        this.NextQuestButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) 
            {
                if (currentQuest == questwin.numOfQuests)
                    currentQuest = 1; else currentQuest++;
                LoadQuest(currentQuest);
            }
        });
        
        // projection modes buttons
        this.PerspecButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                panel3D.engine.mode = Engine.Engine.MODE_PERSPECTIVE;
                panel3D.repaint();
            }
        });
        this.OrthoButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                panel3D.engine.mode = Engine.Engine.MODE_ORTHO;
                panel3D.repaint();
            }
        });
        
        this.ShowErrorsBox.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                CheckField();
            }
        });
        
        // help window
        this.HelpButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                JOptionPane.showMessageDialog(null, 
                        "Following keys are available: " + System.getProperty("line.separator") + 
                        "A: Rotate the gamefield left" + System.getProperty("line.separator") +
                        "S: Rotatet the gamefield right" + System.getProperty("line.separator") +
                        "Arrow UP: Move Camera up" + System.getProperty("line.separator") +
                        "Arrow DOWN: Move Camera down" + System.getProperty("line.separator") +
                        "Arrow LEFT: Move Camera left" + System.getProperty("line.separator") +
                        "Arrow RIGHT: Move Camera right" + System.getProperty("line.separator") +
                        "M: Move Camera up" + System.getProperty("line.separator") +
                        "N: Move Camera down" + System.getProperty("line.separator") +
                        "Note: Make sure that 3D window has the focus by doubleclicking it.",  
                        "Keyboard Handling",
                        JOptionPane.INFORMATION_MESSAGE);
            }
        });

    }
    
    /*
     * Called from LoadQuestFrame. Load new quest.
     **/
    public void LoadQuest(int questNum)
    {
        if (quest.LoadQuestFromFile("Quests/" + questNum + ".txt")) 
        {
            this.currentQuest = questNum;
            this.getAppletContext().showStatus("Quest loaded: " + questNum);
            this.SolvedLabel.setText("");
            // tell gamefield the new quest
            field.setField(quest.field);
            field.repaint();
            // tell our 3d view the new quest
            panel3D.setNewQuest(quest.field);
            bFileLoaded = true;
        }
        quest.PrintField();
    }
    
    /*
     * Called from gamefield if quest was solved.
     */
    public void QuestSolved()
    {
        this.SolvedLabel.setText("Quest solved!!!");
    }
    
    /*
     * Update 3D view if user made a change to the gamefield. Called from gamefield.
     */
    void update3DView(int cellx, int celly, byte value) 
    {
        
        //panel3D.updateCell(cellx, celly, value);  // DOES NOT WORK !
        
        /* Every time the user makes an input the complete 3d wireframe world is rebuild...
         * That's a no-go but the simplest way cause the updateCell does not work till now.
         */
        panel3D.setNewQuest(field.game);
    }
    
    public void CheckField()
    {
        field.toggleErrors();
    }
    
    /** This method is called from within the init() method to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        buttonGroup1 = new javax.swing.ButtonGroup();
        LoadQuestButton = new javax.swing.JButton();
        SolvedLabel = new javax.swing.JLabel();
        jSeparator1 = new javax.swing.JSeparator();
        jLabel1 = new javax.swing.JLabel();
        RestartButton = new javax.swing.JButton();
        NextQuestButton = new javax.swing.JButton();
        OrthoButton = new javax.swing.JRadioButton();
        PerspecButton = new javax.swing.JRadioButton();
        jLabel2 = new javax.swing.JLabel();
        HelpButton = new javax.swing.JButton();
        ShowErrorsBox = new javax.swing.JCheckBox();

        getContentPane().setLayout(null);

        LoadQuestButton.setText("Load Quest...");
        LoadQuestButton.setFocusPainted(false);
        getContentPane().add(LoadQuestButton);
        LoadQuestButton.setBounds(10, 10, 110, 23);

        SolvedLabel.setFont(new java.awt.Font("Arial", 1, 18));
        SolvedLabel.setForeground(new java.awt.Color(0, 204, 0));
        getContentPane().add(SolvedLabel);
        SolvedLabel.setBounds(120, 350, 190, 20);

        jSeparator1.setOrientation(javax.swing.SwingConstants.VERTICAL);
        getContentPane().add(jSeparator1);
        jSeparator1.setBounds(320, 10, 2, 400);

        jLabel1.setText("3D Wireframe View");
        getContentPane().add(jLabel1);
        jLabel1.setBounds(335, 10, 130, 14);

        RestartButton.setText("Restart");
        RestartButton.setFocusPainted(false);
        getContentPane().add(RestartButton);
        RestartButton.setBounds(140, 10, 110, 23);

        NextQuestButton.setText(">> Go on with next quest");
        getContentPane().add(NextQuestButton);
        NextQuestButton.setBounds(70, 390, 180, 23);

        buttonGroup1.add(OrthoButton);
        OrthoButton.setText("Orthogonal");
        OrthoButton.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        OrthoButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        getContentPane().add(OrthoButton);
        OrthoButton.setBounds(340, 390, 85, 15);

        buttonGroup1.add(PerspecButton);
        PerspecButton.setSelected(true);
        PerspecButton.setText("Perspective");
        PerspecButton.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        PerspecButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        getContentPane().add(PerspecButton);
        PerspecButton.setBounds(340, 370, 85, 15);

        jLabel2.setText("Camera Mode:");
        getContentPane().add(jLabel2);
        jLabel2.setBounds(330, 350, 100, 14);

        HelpButton.setFont(new java.awt.Font("Arial", 0, 8));
        HelpButton.setText("?");
        getContentPane().add(HelpButton);
        HelpButton.setBounds(500, 10, 39, 19);

        ShowErrorsBox.setText("Show errors");
        ShowErrorsBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        ShowErrorsBox.setFocusable(false);
        ShowErrorsBox.setMargin(new java.awt.Insets(0, 0, 0, 0));
        getContentPane().add(ShowErrorsBox);
        ShowErrorsBox.setBounds(10, 350, 100, 15);

    }// </editor-fold>//GEN-END:initComponents
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton HelpButton;
    private javax.swing.JButton LoadQuestButton;
    private javax.swing.JButton NextQuestButton;
    private javax.swing.JRadioButton OrthoButton;
    private javax.swing.JRadioButton PerspecButton;
    private javax.swing.JButton RestartButton;
    private javax.swing.JCheckBox ShowErrorsBox;
    private javax.swing.JLabel SolvedLabel;
    private javax.swing.ButtonGroup buttonGroup1;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JSeparator jSeparator1;
    // End of variables declaration//GEN-END:variables
    
}
